c chi2_func.f
c Run integrator and calculate chi^2 for a given starting vector x().
c Miroslav Broz (miroslav.broz@email.cz), May 26th 2016

      real*8 function chi2_func(x)

      include '../swift.inc'
      include '../misc/const.inc'
      include '../tides/tides.inc'
      include 'simplex.inc'
      include 'dependent.inc'
      real*8 x(ndim)

c variables for numerical integration
      integer NOUT1, NOUT2, NOUT
      real*8 m(NBODMAX), r(NBODMAX,3), v(NBODMAX,3)
      real*8 elmts(NBODMAX,6), rj(NBODMAX,3), vj(NBODMAX,3)
      real*8 tout1(OUTMAX1), tout2(OUTMAX2), tout(OUTMAX)
      real*8 rout1(OUTMAX1,NBODMAX,3), vout1(OUTMAX1,NBODMAX,3),
     :       rout2(OUTMAX2,NBODMAX,3), vout2(OUTMAX2,NBODMAX,3)
      real*8 rb(OUTMAX,NBODMAX,3), vb(OUTMAX,NBODMAX,3),
     :       rh(OUTMAX,NBODMAX,3), vh(OUTMAX,NBODMAX,3),
     :       rp(OUTMAX,NBODMAX,3), vp(OUTMAX,NBODMAX,3),
     :       rp3(OUTMAX,NBODMAX,3), vp3(OUTMAX,NBODMAX,3)
      character*80 inparfile

c internal variables
      integer i, j, k, l, i1st, iu, ialpha, nmin
      real*8 chi2
      real*8 r_photocentre(3), v_photocentre(3), Lumtot
      real*8 lambda_eff, band_eff
      real*8 msum, tmp
      real*8 tmin(MINMAX), duration(MINMAX)  ! from chi2_func_TTV.f
      real*8 gamma, gamma_auday
      integer n_of_interest
      real*8 t_of_interest(TIMEMAX)
      real*8 t_of_interest1(TIMEMAX), t_of_interest2(TIMEMAX)

c functions
      real*8 merit_func, kms_auday

      data iu /10/
      data i1st /0/
      save i1st, n_of_interest, t_of_interest

c-----------------------------------------------------------------------
c
c get both free and fixed parameters
c
      j = 0
      do i = 1, nparam
        if (variable(i)) then
          j = j+1
          x_param(i) = x(j)
        endif
      enddo

      write(*,30) "# x() array:"
30    format(a,$)
      write(*,*) (x(i), i=1,ndim)

c create m(), elmts() arrays for easy manipulation

      do i = 1, nbod
        j = i
        m(i) = x_param(j)*GM_S
      enddo

      do i = 2, nbod
        do k = 1, 6
          j = nbod+6*(i-2)+k
          elmts(i,k) = x_param(j)
        enddo
        do k = 3, 6
          elmts(i,k) = elmts(i,k)*deg
        enddo
      enddo

      do i = 1, nbod
        j = nbod+6*(nbod-1)+i
        T_eff(i) = x_param(j)
      enddo
      do i = 1, nbod
        j = nbod+6*(nbod-1)+nbod+i
        R_star(i) = x_param(j)
      enddo

      gamma = x_param(nparam-1)
      d_pc = x_param(nparam)

c compute luminosities (for photocentre computations)
      lambda_eff = 550.d-9
      band_eff = 88.d-9
      call luminosity(T_eff, R_star, nbod, lambda_eff, band_eff,
     :  Lum, Lumtot)

      if (debug_swift) then
        do i = 1, nbod
          write(*,*) '# Lum(', i, ') = ', Lum(i)/Lumtot
        enddo
      endif

c compute jacobian coordinates
      msum = m(1)
      ialpha = -1
      do i = 2, nbod
        msum = msum + m(i)
        call orbel_el2xv(msum,ialpha,
     :    elmts(i,1),elmts(i,2),elmts(i,3),
     :    elmts(i,4),elmts(i,5),elmts(i,6),
     :    rj(i,1),rj(i,2),rj(i,3),
     :    vj(i,1),vj(i,2),vj(i,3))

c adjust coordinates (the elements were standard stellar-astronomy)
        tmp = rj(i,1)
        rj(i,1) = -rj(i,2)
        rj(i,2) = tmp
        rj(i,3) = -rj(i,3)
        tmp = vj(i,1)
        vj(i,1) = -vj(i,2)
        vj(i,2) = tmp
        vj(i,3) = -vj(i,3)
      enddo

c convert to barycentric frame

      call coord_j2b(nbod,m,
     :  rj(1,1),rj(1,2),rj(1,3),vj(1,1),vj(1,2),vj(1,3),
     :  r(1,1),r(1,2),r(1,3),v(1,1),v(1,2),v(1,3))

      if (debug_swift) then
        write(*,*) "# barycentric coordinates:"
        do i = 1, nbod
          write(*,*) m(i)/GM_S,(r(i,k), k=1,3), (v(i,k), k=1,3)
        enddo
      endif

c-----------------------------------------------------------------------
c
c init tidal code
c
      if (i1st.eq.0) then
        call io_init_tides("tides.in", nbod)
        i1st = 1
      endif

      do i = 1, nbod
        R_body(i) = R_star(i)*R_S/AU
      enddo

c-----------------------------------------------------------------------
c
c read all times of interest
c

      call read_time_all(n_of_interest, t_of_interest)

      do i = 1, n_of_interest
        t_of_interest1(i) = t_of_interest(i)-T0
        j = n_of_interest-i+1
        t_of_interest2(j) = T0-t_of_interest(i)
      enddo

c-----------------------------------------------------------------------
c
c forward integration of orbits
c
      inparfile = "param.in"
      call swift_bs_xyzb(nbod,m,r,v,NOUT1,OUTMAX1,tout1,rout1,vout1,
     :  inparfile,eps_BS,debug,n_of_interest,t_of_interest1)

c backward integration (i.e., with reversed velocities)

      do i = 1, nbod
        do j = 1, 3
          v(i,j) = -v(i,j)
        enddo
      enddo

      inparfile = "param_BACK.in"
      call swift_bs_xyzb(nbod,m,r,v,NOUT2,OUTMAX2,tout2,rout2,vout2,
     :  inparfile,eps_BS,debug,n_of_interest,t_of_interest2)

c merge the integration output and add Julian date

      NOUT = NOUT1+NOUT2
      if (NOUT.gt.OUTMAX) then
        write(*,*) "chi2_func: Error number of output data ",
     :    "> OUTMAX = ", OUTMAX
        stop
      endif

      do i = 1, NOUT2
        l = NOUT2-i+1
        tout(l) = T0 - tout2(i)
        do j = 1, nbod
          do k = 1,3
            rb(l,j,k) = rout2(i,j,k)
            vb(l,j,k) = -vout2(i,j,k)
          enddo
        enddo
      enddo

      do i = 1, NOUT1
        l = NOUT2+i
        tout(l) = T0 + tout1(i)
        do j = 1, nbod
          do k = 1,3
            rb(l,j,k) = rout1(i,j,k)
            vb(l,j,k) = vout1(i,j,k)
          enddo
        enddo
      enddo

c add systemic velocity

      gamma_auday = kms_auday(gamma)
      do i = 1, NOUT
        do j = 1, nbod
          vb(i,j,3) = vb(i,j,3) + gamma_auday
        enddo
      enddo

      if (debug_swift) then
        open(unit=iu,file="out_JDATE_barycentric.dat",status="unknown")
        do i = 1, NOUT
          do j = 1, nbod
            write(iu,*) tout(i),-j,rb(i,j,1),rb(i,j,2),rb(i,j,3),
     :        vb(i,j,1),vb(i,j,2),vb(i,j,3)
          enddo
        enddo
        close(iu)
      endif

c convert to heliocentric (1-centric) coordinates

      do i = 1, NOUT
        do j = 1, nbod
          do k = 1, 3
            rh(i,j,k) = rb(i,j,k) - rb(i,1,k)
            vh(i,j,k) = vb(i,j,k) - vb(i,1,k)
          enddo
        enddo
      enddo

      if (debug_swift) then
        open(unit=iu,file="out_JDATE_heliocentric.dat",status="unknown")
        do i = 1, NOUT
          do j = 1, nbod
            write(iu,*) tout(i),-j,rh(i,j,1),rh(i,j,2),rh(i,j,3),
     :        vh(i,j,1),vh(i,j,2),vh(i,j,3)
          enddo
        enddo
        close(iu)
      endif

c convert to photocentric (1+2-centric) coordinates

      do i = 1, NOUT
        do k = 1, 3
          Lumtot = 0.d0
          r_photocentre(k) = 0.d0
          v_photocentre(k) = 0.d0
          do j = 1, min(2, nbod)
             Lumtot = Lumtot + Lum(j)
             r_photocentre(k) = r_photocentre(k) + rb(i,j,k)*Lum(j)
             v_photocentre(k) = v_photocentre(k) + vb(i,j,k)*Lum(j)
          enddo
          r_photocentre(k) = r_photocentre(k)/Lumtot
          v_photocentre(k) = v_photocentre(k)/Lumtot
        enddo
        do j = 1, nbod
          do k = 1, 3
            rp(i,j,k) = rb(i,j,k) - r_photocentre(k)
            vp(i,j,k) = vb(i,j,k) - v_photocentre(k)
          enddo
        enddo
      enddo

      if (debug_swift) then
        open(unit=iu,file="out_JDATE_photocentric.dat",status="unknown")
        do i = 1, NOUT
          do j = 1, nbod
            write(iu,*) tout(i),-j,rp(i,j,1),rp(i,j,2),rp(i,j,3),
     :        vp(i,j,1),vp(i,j,2),vp(i,j,3)
          enddo
        enddo
        close(iu)
      endif

c convert to yet another photocentric (1+2+3-centric) coordinates

      do i = 1, NOUT
        do k = 1, 3
          Lumtot = 0.d0
          r_photocentre(k) = 0.d0
          v_photocentre(k) = 0.d0
          do j = 1, min(3, nbod)
             Lumtot = Lumtot + Lum(j)
             r_photocentre(k) = r_photocentre(k) + rb(i,j,k)*Lum(j)
             v_photocentre(k) = v_photocentre(k) + vb(i,j,k)*Lum(j)
          enddo
          r_photocentre(k) = r_photocentre(k)/Lumtot
          v_photocentre(k) = v_photocentre(k)/Lumtot
        enddo
        do j = 1, nbod
          do k = 1, 3
            rp3(i,j,k) = rb(i,j,k) - r_photocentre(k)
            vp3(i,j,k) = vb(i,j,k) - v_photocentre(k)
          enddo
        enddo
      enddo

      if (debug_swift) then
        open(unit=iu,file="out_JDATE_photocentric3.dat",
     :    status="unknown")
        do i = 1, NOUT
          do j = 1, nbod
            write(iu,*) tout(i),-j,rp3(i,j,1),rp3(i,j,2),rp3(i,j,3),
     :        vp3(i,j,1),vp3(i,j,2),vp3(i,j,3)
          enddo
        enddo
        close(iu)
      endif

c-----------------------------------------------------------------------
c
c calculate the chi^2 values
c
c   1. speckle-interferometry data
c   2. radial-velocity data
c   3. transit-timing variations
c   4. eclipse durations
c   5. interferometric visibilities
c   6. interferometric closure phases
c   7. interferometric triple product amplitude
c   8. light curve
c   9. synthetic spectra
c
      call chi2_func_SKY(NOUT, tout, rp, rp3, chi2_SKY, n_SKY)

      call chi2_func_RV(NOUT, tout, vb, chi2_RV, n_RV)

      call chi2_func_TTV(NOUT, NOUT2, m, tout, rh, rb, nmin, tmin,
     :  duration, chi2_TTV, n_TTV)

      call chi2_func_ECL(nmin, tmin, duration, chi2_ECL, n_ECL)

      call chi2_func_VIS(NOUT, tout, rh, chi2_VIS, n_VIS)

      call chi2_func_CLO(NOUT, tout, rh, chi2_CLO, n_CLO)

      call chi2_func_T3(chi2_T3, n_T3)

      call chi2_func_LC(NOUT, NOUT2, m, tout, rh, vh, rb, chi2_LC, n_LC)

      call chi2_func_SYN(NOUT, tout, vb, chi2_SYN, n_SYN)

c-----------------------------------------------------------------------
c
c add an artificial term to constrain the masses!
c
      chi2_MASS = 0.d0
      do i = 1, nbod
        j = i  ! in x_param()
        chi2_MASS = chi2_MASS + merit_func(x_param(j),m_min(i),m_max(i))
      enddo

c-----------------------------------------------------------------------

c sum the results

      n_fit = n_SKY + n_RV + n_TTV + n_ECL + n_VIS + n_CLO + n_T3 + n_LC
     :  + n_SYN
      chi2 = w_SKY*chi2_SKY + w_RV*chi2_RV + w_TTV*chi2_TTV
     :  + w_ECL*chi2_ECL + w_VIS*chi2_VIS + w_CLO*chi2_CLO
     :  + w_T3*chi2_T3 + w_LC*chi2_LC + w_SYN*chi2_SYN + chi2_MASS

      write(*,30) "# chi^2 value: "
      write(*,*)
     :  n_SKY, n_RV, n_TTV, n_ECL, n_VIS, n_CLO, n_T3, n_LC, n_SYN,
     :  n_fit, chi2_SKY, chi2_RV, chi2_TTV, chi2_ECL, chi2_VIS,
     :  chi2_CLO, chi2_T3, chi2_LC, chi2_SYN, chi2_MASS, chi2

c write hi-precision output

      open(unit=iu, file="chi2_func.tmp", access="append")
      write(iu,*) (x_param(j), j=1,nparam),
     :  n_SKY, n_RV, n_TTV, n_ECL, n_VIS, n_CLO, n_T3, n_LC, n_SYN,
     :  n_fit, chi2_SKY, chi2_RV, chi2_TTV, chi2_ECL, chi2_VIS,
     :  chi2_CLO, chi2_T3, chi2_LC, chi2_SYN, chi2_MASS, chi2
      close(iu)

      chi2_func = chi2

      return
      end


